<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="LokalizeMainWindow" version="9">
<MenuBar>
  <Menu name="file"><text>&amp;File</text>
    <Action name="file_new"/>
    <MergeLocal name="new_merge"/>
    <Action name="file_open"/>
    <Action name="file_open_recent"/>
    <MergeLocal name="open_merge"/>
    <Separator/>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
  </Menu>

  <Merge/>
  <Menu name="project"><text>&amp;Project</text>
    <Action name="project_overview"/>
    <Action name="project_configure"/>
    <Separator/>
    <Action name="project_open"/>
    <!--<ActionList name="project_actions" />-->
    <Action name="project_open_recent"/>
    <Action name="project_close"/>
    <Separator/>
    <Action name="project_create"/>
    <Action name="project_create_odf"/>
    <Merge/>
    <!--<Separator/>
    <Menu name="project_actions"><text>&amp;Scripts</text>
    </Menu>-->
  </Menu>
  <Menu name="tools">
    <MergeLocal name="tools_spelling_merge"/>
    <Separator/>
    <Action name="tools_filesearch"/>
    <Action name="tools_filesearch_next"/>
    <Separator/>
    <Action name="tools_widgettextcapture"/>
    <Separator/>
    <Action name="tools_glossary"/>
    <Separator/>
    <Action name="tools_tm"/>
    <Action name="tools_tm_manage"/>
    <Merge/>
  </Menu>
  <Menu name="settings">
    <Menu name="toolviews"><text>Tool&amp;views</text>
    </Menu>
    <Merge/>
  </Menu>
</MenuBar>
<!--<State name="po" >
  <disable>
  </disable>
</State>
<State name="xliff" >
  <enable>
  </enable>
</State>-->
</gui>
