<!DOCTYPE KrossScripting SYSTEM "kpartgui.dtd">
<KrossScripting>
	<collection name="tools" text="Tools">
		<script text="Merge into ODF" comment="Generates translated version of ODF document"
			name="xliff2odf"
			interpreter="python"
                        enabled="true"
			file="xliff2odf.py" />
	</collection>
</KrossScripting>
