<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui name="FileSearch" version="0">
<MenuBar>
  <Menu name="tools">
    <Action name="showfilelist_action"/>
    <Separator/>
    <MergeLocal/>
  </Menu>
</MenuBar>
<ToolBar name="mainToolBar"><text>Main Toolbar</text>
  <Separator/>
  <Action name="showfilelist_action"/>
  <Action name="showmassreplace_action"/>
  <Action name="showqa_action"/>
</ToolBar>
</gui>
