/*
  This file is part of KAider
  This file is based on the one from KBabel

  SPDX-FileCopyrightText: 2002-2003 Stanislav Visnovsky <visnovsky@kde.org>
  SPDX-FileCopyrightText: 2007 Nick Shaforostoff <shafff@ukr.net>
  SPDX-FileCopyrightText: 2018-2019 Simon Depiets <sdepiets@gmail.com>

  SPDX-License-Identifier: GPL-2.0-or-later WITH LicenseRef-Qt-Commercial-exception-1.0
*/

#include "catalogfileplugin.h"

using namespace GettextCatalog;

CatalogExportPlugin::CatalogExportPlugin()
//    : d(new CatalogExportPluginPrivate)
{
}

CatalogExportPlugin::~CatalogExportPlugin()
{
//    delete d;
}
